/* Main program for all test programs */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <err.h>
#include <errno.h>
#include <sys/wait.h>
#include <sys/stat.h>

#include "stress.h"

volatile int done_testing;
char *home;

static	pid_t *r;

void
handler(int i)
{
	int j;

	done_testing = 1;
	for (j = 0; j < op->incarnations; j++) {
		if (op->verbose > 2)
			printf("handler: kill -HUP %d\n", r[j]);
		if (kill(r[j], SIGHUP) == -1)
			if (errno != ESRCH)
				warn("kill(%d, SIGHUP), %s:%d", r[j], __FILE__, __LINE__);
	}
	if (op->kill == 1) {
		sleep(5);
		/* test programs may have blocked for the SIGHUP, so try harder */
		for (j = 0; j < op->incarnations; j++) {
			if (op->verbose > 2)
				printf("handler: kill -KILL %d\n", r[j]);
			(void) kill(r[j], SIGKILL);
		}
	}
}

void
run_test_handler(int i)
{

	done_testing = 1;
}

static void
run_tests(int i)
{
	signal(SIGHUP, run_test_handler);
	arc4random_stir();
	if ((strcmp(getprogname(), "run") != 0) && (op->nodelay == 0))
		sleep(random_int(1,10));
	setup(i);
	while (done_testing == 0) {
		test();
	}
	cleanup();
	exit(EXIT_SUCCESS);
}

static void
run_incarnations(void)
{
	int i;
	int s;

	signal(SIGHUP, handler);
	for (i = 0; i < op->incarnations && done_testing == 0; i++) {
		if ((r[i] = fork()) == 0) {
			run_tests(i);
		}
		if (r[i] < 0) {
			err(1, "fork(), %s:%d", __FILE__, __LINE__);
			break;
		}
	}
	for (i = 0; i < op->incarnations; i++)
		if (waitpid(r[i], &s, 0) == -1)
			err(1, "waitpid(%d), %s:%d", r[i], __FILE__, __LINE__);

	exit(EXIT_SUCCESS);
}

static int
run_test(void)
{
	pid_t p;
	time_t start;
	int status = 0;

	if (random_int(1,100) > op->load)
		return (status);

	start = time(NULL);
	done_testing = 0;
	fflush(stdout);
	p = fork();
	if (p == 0)
		run_incarnations();
	if (p < 0)
		err(1, "fork() in %s:%d", __FILE__, __LINE__);
	while (done_testing != 1 &&
			(time(NULL) - start) < op->run_time)
		sleep(1);
	if (kill(p, SIGHUP) == -1)
		warn("kill(%d, SIGHUP), %s:%d", p, __FILE__, __LINE__);

	if (waitpid(p, &status, 0) == -1)
		err(1, "waitpid(%d), %s:%d", p, __FILE__, __LINE__);

	return (status);
}

int
main(int argc, char **argv)
{
	struct stat sb;
	int status = 0;

	options(argc, argv);

	umask(0);
	if (stat(op->wd, &sb) == -1) {
		if (mkdir(op->wd, 0770) == -1)
			err(1, "mkdir(%s) %s:%d", op->wd, __FILE__, __LINE__);
	}
	if ((home = getcwd(NULL, 0)) == NULL)
		err(1, "getcwd(), %s:%d",  __FILE__, __LINE__);
	if (chdir(op->wd) == -1)
		err(1, "chdir(%s) %s:%d", op->wd, __FILE__, __LINE__);

	r = (pid_t *)malloc(op->incarnations * sizeof(pid_t));

	status = run_test();

	return (status);
}
