#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sysexits.h>
#include <string.h>

#include "stress.h"

opt_t opt;
opt_t *op;

static void
usage(char *where)
{
	char *help;

	if (where != NULL)
		printf("Error in \"%s\"\n", where);
	fprintf(stderr, "Usage: %s [-t | -l | -i | -d | -h | -k | -v]\n", getprogname());
	help = 	"	t <number><s|m|h|d>	: time to run test\n"
		"	l <pct>			: load factor 10 - 100%\n"
		"	i <number>		: max # of parallel incarnations\n"
		"	d <path>		: working directory\n"
		"	h			: hog resources\n"
		"	k			: terminate with SIGHUP + SIGKILL\n"
		"       n                       : no startup delay\n"
		"	v			: verbose\n";
	printf(help);
	exit(EX_USAGE);
}

static int
time2sec(const char *string)
{
	int r, s = 0;
	char modifier;
	r = sscanf(string, "%d%c", &s, &modifier);
	if (r == 2)
		switch(modifier) {
		case 's': break;
		case 'm': s = s * 60; break;
		case 'h': s = s * 60 * 60; break;
		case 'd': s = s * 60 * 60 * 24; break;
		default:
			usage("-t");
		}
	else
		usage("-t");
	return (s);
}

static char *gete(char *name)
{
	char *cp;
	char help[128];

	snprintf(help, sizeof(help), "%s%s", getprogname(), name);
	cp = getenv(help);
	if (cp == NULL)
		cp = getenv(name);
	return (cp);
}

static void
environment(void)
{
	char *cp;

	if ((cp = gete("INCARNATIONS")) != NULL) {
		if (sscanf(cp, "%d", &op->incarnations) != 1)
			usage("INCARNATIONS");
	}
	if ((cp = gete("LOAD")) != NULL) {
		if (sscanf(cp, "%d", &op->load) != 1)
			usage("LOAD");
	}
	if ((cp = gete("RUNTIME")) != NULL) {
		op->run_time = time2sec(cp);
	}
	if ((cp = gete("WD")) != NULL) {
		op->wd = cp;
	}
	if ((cp = gete("HOG")) != NULL) {
		op->hog = 1;
	}
	if ((cp = gete("KILL")) != NULL) {
		op->kill = 1;
	}
	if ((cp = gete("NODELAY")) != NULL) {
		op->nodelay = 1;
	}
	if ((cp = gete("VERBOSE")) != NULL) {
		if (sscanf(cp, "%d", &op->verbose) != 1)
			usage("VERBOSE");
	}
}
	
void
options(int argc, char **argv)
{
	int ch;

	op = &opt;

	op->run_time	= 60;
	op->load	= 100;
	op->wd		= "/tmp/stressX";
	op->incarnations	= 1;
	op->hog		= 0;
	op->kill	= 0;
	op->nodelay	= 0;
	op->verbose	= 0;

	environment();

	while ((ch = getopt(argc, argv, "t:l:i:d:hknv")) != -1)
		switch(ch) {
		case 't':	/* run time */
			op->run_time = time2sec(optarg);
			break;
		case 'l':	/* load factor in pct */
			if (sscanf(optarg, "%d", &op->load) != 1)
				usage("-l");
			break;
		case 'i':	/* max incarnations */
			if (sscanf(optarg, "%d", &op->incarnations) != 1)
				usage("-i");
			break;
		case 'd':	/* working directory */
			op->wd = strdup(optarg);
			break;
		case 'h':	/* hog flag */
			op->hog = 1;
			break;
		case 'k':	/* kill flag */
			op->kill = 1;
			break;
		case 'n':	/* no delay flag */
			op->nodelay = 1;
			break;
		case 'v':	/* verbose flag */
			op->verbose += 1;
			break;
		default:
			usage(NULL);
		}
	op->argc = argc -= optind;
	op->argv = argv += optind;

	if (op->incarnations < 1)
		op->incarnations = 1;
	op->incarnations = random_int(1, op->incarnations);
	if (op->run_time < 15)
		op->run_time = 15;
	if (op->load < 1 || op->load > 100)
		op->load = 100;

	if (op->verbose > 0)
		printf("%8s: run time %5d, incarnations %2d, load %3d, verbose %d\n",
			getprogname(), op->run_time, op->incarnations, op->load,
			op->verbose);
}
