#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <sys/types.h>
#include <sys/sysctl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <err.h>

#include "stress.h"

static unsigned long size;

int
setup(int nb)
{
	unsigned long minsize, maxsize;
	unsigned long usermem;
	unsigned long swapinfo = 0;
	FILE *fd;

	size_t nlen = sizeof(usermem);

	if (sysctlbyname("hw.usermem", &usermem, &nlen, NULL, 0) == -1)
		err(1, "sysctlbyname() %s:%d", __FILE__, __LINE__);
	if ((fd = popen("/usr/sbin/swapinfo -k | tail -1 | awk '{print $2}'", "r")) == NULL)
		err(1, "popen");
	if (fscanf(fd, "%ld", &swapinfo) != 1)
		err(1, "fscanf() %s:%d", __FILE__, __LINE__);
	if (pclose(fd) == -1)
		err(1, "pclose");
	if (swapinfo == 1)
		swapinfo = 0;
	else
		swapinfo = swapinfo * 1024;
	if (op->verbose > 2)
		printf("hw.usermem %ld, swapinfo %ld\n", usermem, swapinfo);

	if (op->hog == 0)
		minsize = usermem * 0.4;
	else
		minsize = usermem * 0.8;
	maxsize = usermem + swapinfo * 0.2;

	size = random_int(minsize, maxsize);
	size = size / op->incarnations;
	if (op->verbose > 1 && nb == 0)
		printf("setup: pid %d, minsize %ldMb, maxsize %ldMb, "
				"size %ldMb (total %ldMb)\n", getpid(), 
			minsize/1024/1024, maxsize/1024/1024, size/1024/1024,
			size * op->incarnations / 1024 / 1024);
	return (0);
}

void
cleanup(void)
{
}

int
test(void)
{
	char *c;
	int i, page;

	c = malloc(size);
	if (c == NULL)	/* check ulimit -d */
		err(1, "malloc(%ld), %s:%d", size, __FILE__, __LINE__);
	page = getpagesize();
	while (done_testing == 0) {
		i = 0;
		while (i < size && done_testing == 0) {
			c[i] = 0;	/* Go easy on crash dump compression */
			i += page;
		}
	}
	free(c);

	return (0);
}
